"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
exports.getReferenceExpressions = exports.getAttributeValueNodes = void 0;
const context_1 = require("../../context");
const templates_1 = require("../../../utils/templates");
function getAttributeValueNodes(element, name, context) {
    const results = [];
    const { startTag } = element;
    for (const attr of startTag.attributes) {
        if (!templates_1.isVDirective(attr)) {
            const { key, value } = attr;
            if (value == null) {
                continue;
            }
            if (key.name === name) {
                results.push(value);
            }
        }
        else {
            const { key, value } = attr;
            if (value == null) {
                continue;
            }
            if (!templates_1.isVBind(key)) {
                continue;
            }
            const bindArg = templates_1.getArgument(key);
            if (bindArg == null) {
                return null;
            }
            if (bindArg !== name) {
                continue;
            }
            const { expression } = value;
            if (expression == null) {
                continue;
            }
            const expressions = getReferenceExpressions(expression, context);
            if (!expressions) {
                return null;
            }
            for (const e of expressions) {
                results.push(e);
            }
        }
    }
    return results;
}
exports.getAttributeValueNodes = getAttributeValueNodes;
function getReferenceExpressions(expression, context) {
    if (expression.type !== "Identifier") {
        return [expression];
    }
    const vueComponent = context_1.getVueComponentContext(context);
    if (!vueComponent) {
        return null;
    }
    const props = vueComponent.findVueComponentProperty(expression.name);
    if (props == null) {
        return null;
    }
    return props;
}
exports.getReferenceExpressions = getReferenceExpressions;
